package dev.ninjdai.werewolf;

import dev.ninjdai.werewolf.uhc.UHCPlayer;
import dev.ninjdai.werewolf.uhc.events.TimedPlayerDeath;
import io.github.togar2.pvp.events.EntityPreDeathEvent;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.entity.ItemEntity;
import net.minestom.server.entity.Player;
import net.minestom.server.event.Event;
import net.minestom.server.event.EventNode;
import net.minestom.server.event.GlobalEventHandler;
import net.minestom.server.event.player.AsyncPlayerConfigurationEvent;
import net.minestom.server.event.player.PlayerBlockBreakEvent;
import net.minestom.server.instance.block.Block;
import net.minestom.server.item.ItemStack;
import net.minestom.server.item.Material;

import java.time.Duration;

public class EventHandler {
    public static void onBlockBreak(PlayerBlockBreakEvent event) {
        Material m = Material.fromNamespaceId(event.getBlock().namespace());
        if (m == null) return;
        ItemEntity item = new ItemEntity(ItemStack.of(m));
        item.setPickupDelay(Duration.ofMillis(500));
        item.setMergeable(true);
        item.setInstance(event.getInstance(), event.getBlockPosition().add(0.5, 0.5, 0.5));
    }

    public static void onPlayerConfiguration(AsyncPlayerConfigurationEvent event) {
        final Player player = event.getPlayer();
        event.setSpawningInstance(Main.overworld);
        player.setRespawnPoint(new Pos(0, 42, 0));
    };

    public static void onPlayerDeath(EntityPreDeathEvent event) {
        UHCPlayer player = (UHCPlayer) event.getEntity();

        event.setCancelDeath(true);
        player.toggleSpectating();

        var timedPlayerDeath = new TimedPlayerDeath(player, event.getDamage(), 15);

        player.getUHCRole().onPlayerDeath(player, timedPlayerDeath);
    }

    public static void register(EventNode<Event> node) {
        node.addListener(PlayerBlockBreakEvent.class, EventHandler::onBlockBreak);
        node.addListener(AsyncPlayerConfigurationEvent.class, EventHandler::onPlayerConfiguration);
        node.addListener(EntityPreDeathEvent.class, event -> {
            if (event.getEntity() instanceof UHCPlayer) onPlayerDeath(event);
        });
    }
}
