package dev.ninjdai.werewolf.commands;

import dev.ninjdai.werewolf.uhc.UHCPlayer;
import dev.ninjdai.werewolf.uhc.roles.Role;
import dev.ninjdai.werewolf.uhc.roles.Roles;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.Style;
import net.kyori.adventure.text.format.TextColor;
import net.kyori.adventure.text.format.TextDecoration;
import net.minestom.server.command.builder.Command;
import net.minestom.server.command.builder.arguments.ArgumentEnum;
import net.minestom.server.command.builder.arguments.ArgumentType;
import net.minestom.server.command.builder.arguments.minecraft.ArgumentEntity;
import net.minestom.server.utils.entity.EntityFinder;

public class LGRoleCommand extends Command {
    static final ArgumentEnum<Roles> roleArgument = ArgumentType.Enum("target_role", Roles.class);
    static final ArgumentEntity singlePlayerArgument = ArgumentType.Entity("player").onlyPlayers(true).singleEntity(true);

    public LGRoleCommand() {
        super("role");

        addSubcommand(new LGRoleAddCommand());
        addSubcommand(new LGRoleGetCommand());

        setDefaultExecutor(((sender, context) -> sender.sendMessage("Please select an action")));
    }

    static class LGRoleAddCommand extends Command {
        public LGRoleAddCommand() {
            super("add");

            addSyntax(((sender, context) -> {
                EntityFinder finder = context.get(singlePlayerArgument);
                UHCPlayer player = (UHCPlayer) finder.findFirstPlayer(sender);

                Role role = context.get(roleArgument).get();
                if (player == null) {
                    sender.sendMessage("Couldn't find player");
                    return;
                }
                player.setUHCRole(role);
                sender.sendMessage(Component.empty().style(Style.style(TextDecoration.ITALIC))
                        .append(Component.text(player.getUsername()))
                        .append(Component.text(" now has role ", TextColor.color(94,94,94)))
                        .append(Component.text(role.getName()))
                );
            }), roleArgument, singlePlayerArgument);
        }
    }

    static class LGRoleGetCommand extends Command {
        public LGRoleGetCommand() {
            super("get");

            addSyntax(((sender, context) -> {
                EntityFinder finder = context.get(singlePlayerArgument);
                UHCPlayer player = (UHCPlayer) finder.findFirstPlayer(sender);
                if (player == null) {
                    sender.sendMessage("Couldn't find player");
                    return;
                }
                Role role = player.getUHCRole();
                sender.sendMessage(Component.empty().style(Style.style(TextDecoration.ITALIC))
                        .append(Component.text(player.getUsername()))
                        .append(Component.text(" has role ", TextColor.color(94,94,94)))
                        .append(Component.text(role.getName()))
                );
            }), singlePlayerArgument);
        }
    }
}
