package dev.ninjdai.werewolf.commands;

import dev.ninjdai.werewolf.uhc.UHCPlayer;
import dev.ninjdai.werewolf.uhc.teams.Team;
import dev.ninjdai.werewolf.uhc.teams.Teams;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.Style;
import net.kyori.adventure.text.format.TextColor;
import net.kyori.adventure.text.format.TextDecoration;
import net.minestom.server.command.builder.Command;
import net.minestom.server.command.builder.arguments.ArgumentEnum;
import net.minestom.server.command.builder.arguments.ArgumentType;
import net.minestom.server.command.builder.arguments.minecraft.ArgumentEntity;
import net.minestom.server.utils.entity.EntityFinder;

public class LGTeamCommand extends Command {
    static final ArgumentEnum<Teams> teamArgument = ArgumentType.Enum("target_team", Teams.class);
    static final ArgumentEntity singlePlayerArgument = ArgumentType.Entity("player").onlyPlayers(true).singleEntity(true);

    public LGTeamCommand() {
        super("team");

        addSubcommand(new LGTeamAddCommand());
        addSubcommand(new LGTeamGetCommand());

        setDefaultExecutor(((sender, context) -> sender.sendMessage("Please select an action")));
    }

    static class LGTeamAddCommand extends Command {
        public LGTeamAddCommand() {
            super("add");

            addSyntax(((sender, context) -> {
                EntityFinder finder = context.get(singlePlayerArgument);
                UHCPlayer player = (UHCPlayer) finder.findFirstPlayer(sender);

                Team team = context.get(teamArgument).get();
                if (player == null) {
                    sender.sendMessage("Couldn't find player");
                    return;
                }
                player.setUHCTeam(team);
                sender.sendMessage(Component.empty().style(Style.style(TextDecoration.ITALIC))
                        .append(Component.text(player.getUsername(), TextColor.color(team.getColor())))
                        .append(Component.text(" added to team ", TextColor.color(94,94,94)))
                        .append(Component.text(team.getName(), TextColor.color(team.getColor())))
                );
            }), teamArgument, singlePlayerArgument);
        }
    }

    static class LGTeamGetCommand extends Command {
        public LGTeamGetCommand() {
            super("get");

            addSyntax(((sender, context) -> {
                EntityFinder finder = context.get(singlePlayerArgument);
                UHCPlayer player = (UHCPlayer) finder.findFirstPlayer(sender);
                if (player == null) {
                    sender.sendMessage("Couldn't find player");
                    return;
                }
                Team team = player.getUHCTeam();
                sender.sendMessage(Component.empty().style(Style.style(TextDecoration.ITALIC))
                        .append(Component.text(player.getUsername(), TextColor.color(team.getColor())))
                        .append(Component.text(" belongs to team ", TextColor.color(94,94,94)))
                        .append(Component.text(team.getName(), TextColor.color(team.getColor())))
                );
            }), singlePlayerArgument);
        }
    }
}
