package dev.ninjdai.werewolf.uhc.events;

import dev.ninjdai.werewolf.uhc.UHCPlayer;
import dev.ninjdai.werewolf.uhc.roles.Role;
import dev.ninjdai.werewolf.uhc.roles.RoleProperties;
import dev.ninjdai.werewolf.uhc.roles.Roles;
import dev.ninjdai.werewolf.uhc.teams.WinCondition;
import net.kyori.adventure.sound.Sound;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.TextColor;
import net.minestom.server.MinecraftServer;
import net.minestom.server.coordinate.Pos;
import net.minestom.server.entity.Entity;
import net.minestom.server.entity.ItemEntity;
import net.minestom.server.entity.Player;
import net.minestom.server.entity.damage.Damage;
import net.minestom.server.instance.Instance;
import net.minestom.server.item.ItemStack;
import net.minestom.server.sound.SoundEvent;
import net.minestom.server.timer.Task;
import net.minestom.server.timer.TaskSchedule;
import org.jetbrains.annotations.NotNull;

import java.time.Duration;
import java.util.Set;

public class TimedPlayerDeath {
    private final ItemStack[] inventory;
    private final Instance instance;
    private final Pos deathPos;
    private final UHCPlayer targetPlayer;
    private final Task triggerTask;
    private final Set<UHCPlayer> playersAtDeath;

    public TimedPlayerDeath(@NotNull Player player, @NotNull Damage damage, int seconds) {
        this.inventory = player.getInventory().getItemStacks();
        this.instance = player.getInstance();
        this.deathPos = player.getPosition();
        this.targetPlayer = (UHCPlayer) player;
        this.playersAtDeath = WinCondition.inGamePlayers(instance);

        this.triggerTask = MinecraftServer.getSchedulerManager().scheduleTask(this::trigger, TaskSchedule.seconds(seconds));
    }

    public void cancel() {
        this.triggerTask.cancel();
    }

    private TaskSchedule trigger(){
        if(!targetPlayer.getUHCRole().getRoleProperties().contains(RoleProperties.NO_DEATH_MESSAGE)) {
            this.instance.sendMessage(Component.text(targetPlayer.getUsername()).append(Component.text(" died")));
        }

        if(WinCondition.checkWin(playersAtDeath)) {
            instance.playSound(Sound.sound().type(SoundEvent.ENTITY_ENDER_DRAGON_GROWL).source(Sound.Source.PLAYER).build());
            instance.sendMessage(Component.empty()
                    .append(Component.text("Team "))
                    .append(Component.text(WinCondition.winningTeam.getName(), TextColor.color(WinCondition.winningTeam.getColor())))
                    .append(Component.text(" wins !"))
            );
            return TaskSchedule.stop();
        }

        for (ItemStack itemStack: inventory) {
            ItemEntity e = new ItemEntity(itemStack);
            e.setPickupDelay(Duration.ofSeconds(1));
            e.setInstance(this.instance, deathPos);
        }

        return TaskSchedule.stop();
    }
}
