package dev.ninjdai.werewolf.uhc.teams;

import lombok.Getter;
import net.minestom.server.color.Color;

import java.util.HashMap;
import java.util.Map;

public class Team {
    public Team(String id, String name, Color color, WinCondition.WinConditionType winConditionType) {
        this.id = id;
        this.name = name;
        this.color = color;
        this.winConditionType = winConditionType;

        teamMap.put(this.id, this);
    }

    public Team(Builder builder){
        this.id = builder.id;
        this.name = builder.name;
        this.color = builder.color;
        this.winConditionType = builder.winConditionType;

        teamMap.put(this.id, this);
    }

    @Getter private String id;
    @Getter private String name;
    @Getter private Color color;
    @Getter private WinCondition.WinConditionType winConditionType;

    // ID: Team
    public static final Map<String, Team> teamMap;
    static {
        teamMap = new HashMap<>();
    };

    public static class Builder {
        private String id = "DUMMY";
        private String name = "Dummy";
        private Color color = new Color(0, 0, 0);
        private WinCondition.WinConditionType winConditionType = WinCondition.WinConditionType.ONLY_TEAM_REMAINING;

        Builder setId(String id) {
            this.id = id;
            return this;
        }

        Builder setName(String name) {
            this.name = name;
            return this;
        }

        Builder setColor(Color color) {
            this.color = color;
            return this;
        }

        Builder setWinConditionType(WinCondition.WinConditionType winConditionType) {
            this.winConditionType = winConditionType;
            return this;
        }

        Team build() {
            return new Team(this);
        }
    }
}
