package dev.ninjdai.werewolf.worldgen;

import de.articdive.jnoise.core.api.functions.Interpolation;
import de.articdive.jnoise.core.api.modifiers.NoiseModifier;
import de.articdive.jnoise.generators.noise_parameters.fade_functions.FadeFunction;
import de.articdive.jnoise.modifiers.clamp.ClampModifier;
import de.articdive.jnoise.modules.octavation.fractal_functions.FractalFunction;
import de.articdive.jnoise.pipeline.JNoise;
import net.minestom.server.coordinate.Point;
import net.minestom.server.instance.block.Block;
import net.minestom.server.instance.generator.GenerationUnit;
import org.jetbrains.annotations.NotNull;

public class LGGenerator {
    static final JNoise noise1 = JNoise.newBuilder()
            .perlin(3301, Interpolation.LINEAR, FadeFunction.QUINTIC_POLY)
            .scale(1/48.0)
            .addModifier(v -> (v + 1) / 2.0)
            .clamp(0.0, 1.0)
            .octavate(3, 0.5, .75, FractalFunction.TURBULENCE, true)
            .build();

    public static void generate(@NotNull GenerationUnit unit) {
        Point start = unit.absoluteStart();
        for (int x = 0; x < unit.size().x(); x++) {
            for (int z = 0; z < unit.size().z(); z++) {
                Point bottom = start.add(x, 0, z);

                synchronized (noise1) {
                    double height = noise1.evaluateNoise(bottom.x(), bottom.z()) * 16 + 20;
                    // * 16 means the height will be between -16 and +16
                    unit.modifier().fill(bottom, bottom.add(1, 0, 1).withY(height), Block.STONE);
                    unit.modifier().fill(bottom.withY(height), bottom.add(1, 0, 1).withY(height+3), Block.DIRT);
                    unit.modifier().fill(bottom.withY(height+3), bottom.add(1, 0, 1).withY(height+4), Block.GRASS_BLOCK);
                }
            }
        }
    }
}
